@@ -9,7 +9,8 @@ import mcp.types as types
 from google.genai import types as genai_types
 
 from hud.tools.computer.settings import computer_settings
-from hud.types import AgentResponse, BaseAgentConfig, MCPToolCall, MCPToolResult
+from hud.tools.native_types import NativeToolSpec
+from hud.types import AgentResponse, AgentType, BaseAgentConfig, MCPToolCall, MCPToolResult
 from hud.utils.types import with_signature
 
 from .base import MCPAgent
@@ -71,6 +72,34 @@ class GeminiCUAAgent(GeminiAgent):
     required_tools: ClassVar[list[str]] = ["gemini_computer"]
     config_cls: ClassVar[type[BaseAgentConfig]] = GeminiCUAConfig
 
+    @classmethod
+    def agent_type(cls) -> AgentType:
+        """Return the AgentType for Gemini CUA."""
+        return AgentType.GEMINI_CUA
+
+    # Legacy tool name patterns for backwards compatibility
+    _LEGACY_COMPUTER_NAMES = ("gemini_computer", "computer_gemini", "computer")
+
+    def _legacy_native_spec_fallback(self, tool: types.Tool) -> NativeToolSpec | None:
+        """Detect Gemini CUA native tools by name for backwards compatibility.
+
+        Supports old environments that expose tools like 'gemini_computer'
+        without native_tools metadata.
+        """
+        name = tool.name
+
+        # Check for computer tool patterns
+        for pattern in self._LEGACY_COMPUTER_NAMES:
+            if name == pattern or name.endswith(f"_{pattern}"):
+                logger.debug("Legacy fallback: detected %s as computer_use tool", name)
+                return NativeToolSpec(
+                    api_type="computer_use",
+                    api_name="gemini_computer",
+                    role="computer",
+                )
+
+        return None
+
     @with_signature(GeminiCUACreateParams)
     @classmethod
     def create(cls, **kwargs: Any) -> GeminiCUAAgent:  # pyright: ignore[reportIncompatibleMethodOverride]
@@ -98,10 +127,17 @@ class GeminiCUAAgent(GeminiAgent):
     def _to_gemini_tool(self, tool: types.Tool) -> genai_types.Tool | None:
         """Convert a single MCP tool to Gemini tool format.
 
-        Handles gemini_computer tool specially by using Gemini's native ComputerUse.
+        Handles computer_use tools specially by using Gemini's native ComputerUse.
+        Uses native_specs metadata first, falls back to name-based detection for
+        old environments.
         """
-        if tool.name == self._computer_tool_name:
-            # Use Gemini's native computer use capability
+        # Check for native spec (new approach) or legacy fallback
+        spec = self.resolve_native_spec(tool)
+
+        if spec and spec.api_type == "computer_use":
+            # This tool should use Gemini's native computer use capability
+            logger.debug("Using native ComputerUse for tool %s (via native_specs)", tool.name)
+            self._computer_tool_name = tool.name  # Track the actual MCP tool name
             return genai_types.Tool(
                 computer_use=genai_types.ComputerUse(
                     environment=genai_types.Environment.ENVIRONMENT_BROWSER,
@@ -109,7 +145,11 @@ class GeminiCUAAgent(GeminiAgent):
                 )
             )
 
-        if tool.name == "computer" or tool.name.endswith("_computer"):
+        # Skip other computer-like tools that don't have native specs for this agent
+        # (they may be meant for other agents like Claude or OpenAI)
+        tool_role = self.get_tool_role(tool)
+        if tool_role == "computer":
+            logger.debug("Skipping computer tool %s (no native_specs for gemini_cua)", tool.name)
             return None
 
         # For non-computer tools, use the parent implementation
