@@ -17,37 +17,15 @@ from typing import Any
 import typer
 import yaml
 
+from hud.cli.utils.environment import find_dockerfile
 from hud.cli.utils.source_hash import compute_source_hash, list_source_files
+from hud.shared.hints import render_hints, secrets_in_build_args
 from hud.utils.hud_console import HUDConsole
 from hud.version import __version__ as hud_version
 
 from .utils.registry import save_to_registry
 
 
-def find_dockerfile(directory: Path) -> Path | None:
-    """Find the Dockerfile in a directory, preferring Dockerfile.hud.
-
-    Checks for Dockerfile.hud first (HUD-specific), then falls back to Dockerfile.
-
-    Args:
-        directory: Directory to search in
-
-    Returns:
-        Path to the Dockerfile if found, None otherwise
-    """
-    # Prefer Dockerfile.hud for HUD environments
-    hud_dockerfile = directory / "Dockerfile.hud"
-    if hud_dockerfile.exists():
-        return hud_dockerfile
-
-    # Fall back to standard Dockerfile
-    standard_dockerfile = directory / "Dockerfile"
-    if standard_dockerfile.exists():
-        return standard_dockerfile
-
-    return None
-
-
 def parse_version(version_str: str) -> tuple[int, int, int]:
     """Parse version string like '1.0.0' or '1.0' into tuple of integers."""
     # Remove 'v' prefix if present
@@ -341,6 +319,52 @@ def parse_base_image(dockerfile_path: Path) -> str | None:
     return None
 
 
+def check_dockerfile_for_secrets(directory: Path, dockerfile: Path) -> list[str]:
+    """Run docker buildx build --check to detect secrets in ARG/ENV.
+
+    Returns a list of variable names that were flagged as potential secrets.
+    This is a fast, non-building lint check.
+    """
+    hud_console = HUDConsole()
+
+    cmd = ["docker", "buildx", "build", "--check"]
+    if dockerfile.name != "Dockerfile":
+        cmd.extend(["-f", str(dockerfile)])
+    cmd.append(str(directory))
+
+    try:
+        result = subprocess.run(  # noqa: S603
+            cmd,
+            capture_output=True,
+            text=True,
+            timeout=60,
+        )
+        output = result.stdout + result.stderr
+
+        pattern = r'Do not use ARG or ENV instructions for sensitive data \((ARG|ENV) "([^"]+)"\)'
+        matches = re.findall(pattern, output)
+
+        if matches:
+            secret_vars = [f"{var_type} {var_name}" for var_type, var_name in matches]
+            return secret_vars
+
+    except subprocess.TimeoutExpired:
+        hud_console.warning("Dockerfile check timed out")
+    except Exception as e:
+        hud_console.debug(f"Dockerfile secrets check failed: {e}")
+
+    return []
+
+
+def display_secrets_warning(secret_vars: list[str]) -> None:
+    """Display a warning about secrets found in Dockerfile ARG/ENV."""
+
+    hud_console = HUDConsole()
+    hud_console.print("")
+    render_hints([secrets_in_build_args(secret_vars)])
+    hud_console.print("")
+
+
 def collect_runtime_metadata(image: str, *, verbose: bool = False) -> dict[str, str | None]:
     """Probe container to capture Python/CUDA/cuDNN/PyTorch versions.
 
@@ -448,13 +472,16 @@ async def analyze_mcp_environment(
     from hud.cli.analyze import parse_docker_command
 
     mcp_config = parse_docker_command(docker_cmd)
+    # Extract server name for display (first key in mcp_config)
+    server_name = next(iter(mcp_config.keys()), None)
 
     # Initialize client and measure timing
-    # Use FastMCP client directly - no mcp_use deprecation warnings
-    from hud.clients.fastmcp import FastMCPHUDClient
+    from fastmcp import Client as FastMCPClient
+
+    from hud.cli.utils.mcp import analyze_environment
 
     start_time = time.time()
-    client = FastMCPHUDClient(mcp_config=mcp_config, verbose=verbose)
+    client = FastMCPClient(transport=mcp_config)
     initialized = False
 
     try:
@@ -462,12 +489,12 @@ async def analyze_mcp_environment(
             hud_console.info("Initializing MCP client...")
 
         # Add timeout to fail fast instead of hanging (60 seconds)
-        await asyncio.wait_for(client.initialize(), timeout=60.0)
+        await asyncio.wait_for(client.__aenter__(), timeout=60.0)
         initialized = True
         initialize_ms = int((time.time() - start_time) * 1000)
 
         # Delegate to standard analysis helper
-        full_analysis = await client.analyze_environment()
+        full_analysis = await analyze_environment(client, verbose, server_name=server_name)
 
         # Normalize and enrich with internalTools if a hub map is present
         tools_list = full_analysis.get("tools", [])
@@ -540,9 +567,9 @@ async def analyze_mcp_environment(
         raise HudException from e
     finally:
         # Only shutdown if we successfully initialized
-        if initialized:
+        if initialized and client.is_connected():
             try:
-                await client.shutdown()
+                await client.close()
             except Exception:
                 # Ignore shutdown errors
                 hud_console.warning("Failed to shutdown MCP client")
@@ -808,6 +835,11 @@ def build_environment(
         )
         hud_console.info("These will be added to the required list in the lock file")
 
+    # Check for secrets in ARG/ENV instructions
+    secret_vars = check_dockerfile_for_secrets(env_dir, dockerfile_path)
+    if secret_vars:
+        display_secrets_warning(secret_vars)
+
     # Check for existing version and increment
     lock_path = env_dir / "hud.lock.yaml"
     existing_version = get_existing_version(lock_path)
