@@ -15,22 +15,23 @@ class TestRunMCPDevServer:
     """Test the main server runner."""
 
     def test_run_dev_server_image_not_found(self) -> None:
-        """Test handling when Docker image doesn't exist."""
-        import click
+        """When using Docker mode without a lock file, exits with typer.Exit(1)."""
+        import typer
 
         with (
-            patch("hud.cli.utils.environment.image_exists", return_value=False),
-            patch("click.confirm", return_value=False),
-            pytest.raises(click.Abort),
+            patch("hud.cli.dev.should_use_docker_mode", return_value=True),
+            patch("hud.cli.dev.Path.cwd"),
+            patch("hud.cli.dev.hud_console"),
+            pytest.raises(typer.Exit),
         ):
             run_mcp_dev_server(
-                module=".",
+                module=None,
                 stdio=False,
                 port=8765,
                 verbose=False,
                 inspector=False,
                 interactive=False,
                 watch=[],
-                docker=False,
+                docker=True,
                 docker_args=[],
             )
