@@ -12,6 +12,7 @@ from __future__ import annotations
 import contextvars
 import logging
 import uuid
+from contextlib import contextmanager
 from typing import TYPE_CHECKING, Any, Self
 
 from hud.environment import Environment
@@ -20,9 +21,11 @@ from hud.shared import make_request
 from hud.telemetry import flush, instrument
 
 if TYPE_CHECKING:
+    from collections.abc import Generator
     from types import TracebackType
 
     from hud.eval.task import Task
+    from hud.tools.types import EvaluationResult
     from hud.types import MCPToolResult
 
 
@@ -58,6 +61,20 @@ def get_current_trace_id() -> str | None:
     return None
 
 
+@contextmanager
+def set_trace_context(trace_id: str) -> Generator[None, None, None]:
+    """Temporarily set trace context from an external trace_id.
+
+    Used by MCP tool handlers to propagate parent trace context into sub-processes.
+    """
+    headers = {"Trace-Id": trace_id}
+    token = _current_trace_headers.set(headers)
+    try:
+        yield
+    finally:
+        _current_trace_headers.reset(token)
+
+
 def get_current_api_key() -> str | None:
     """Get the current API key override from context.
 
@@ -152,6 +169,7 @@ class EvalContext(Environment):
         # User-settable (per-run values, override Environment defaults)
         self.prompt: str | None = None  # From scenario setup or task
         self.reward: float | None = None
+        self.evaluation_result: EvaluationResult | None = None  # Full result with subscores
         self.answer: str | None = None  # Agent's submitted answer
         self.system_prompt: str | None = None  # From task.agent_config, passed to agent
 
@@ -378,9 +396,10 @@ class EvalContext(Environment):
         if self._task is None or self._task.scenario is None:
             return
 
-        reward = await self.run_scenario_evaluate(self._task.scenario)
-        if reward is not None:
-            self.reward = reward
+        result = await self.run_scenario_evaluate(self._task.scenario)
+        if result is not None:
+            self.evaluation_result = result
+            self.reward = result.reward
 
     # =========================================================================
     # Summary Context - Attribute Access Control
@@ -724,4 +743,5 @@ __all__ = [
     "get_current_api_key",
     "get_current_trace_headers",
     "get_current_trace_id",
+    "set_trace_context",
 ]
